==> llm_sandbox-0.3.6-py3-none-any.whl: llm_sandbox/podman.py <==
# ruff: noqa: E501

from typing import TYPE_CHECKING, Any

from podman import PodmanClient

from llm_sandbox.const import SupportedLanguage
from llm_sandbox.core.config import SessionConfig
from llm_sandbox.docker import DockerContainerAPI, SandboxDockerSession
from llm_sandbox.security import SecurityPolicy

if TYPE_CHECKING:
    from podman.domain.images import Image


class PodmanContainerAPI(DockerContainerAPI):
    """Podman implementation of the ContainerAPI protocol."""

    def __init__(self, client: PodmanClient, stream: bool = True) -> None:
        """Initialize Podman container API."""
        self.client = client
        self.stream = stream

    def execute_command(self, container: Any, command: str, **kwargs: Any) -> tuple[int, Any]:
        """Execute command in Podman container."""
        workdir = kwargs.get("workdir")
        exec_kwargs: dict[str, Any] = {
            "cmd": command,
            "stream": self.stream,
            "tty": False,
            "stderr": True,
            "stdout": True,
            "demux": True,
        }
        if workdir:
            exec_kwargs["workdir"] = workdir

        result = container.exec_run(**exec_kwargs)
        # Podman returns (exit_code, output) tuple, Docker returns object with .exit_code and .output
        exit_code, output = result
        return exit_code or 0, output


class SandboxPodmanSession(SandboxDockerSession):
    r"""Sandbox session implemented using Podman containers.

    This class provides a sandboxed environment for code execution by leveraging Podman.
    It inherits from SandboxDockerSession since Podman is designed to be Docker-compatible,
    only overriding the differences in client initialization and API behavior.
    """

    def __init__(
        self,  # NOSONAR
        client: PodmanClient | None = None,
        image: str | None = None,
        dockerfile: str | None = None,
        lang: str = SupportedLanguage.PYTHON,
        keep_template: bool = False,
        commit_container: bool = False,
        verbose: bool = False,
        mounts: list | None = None,
        stream: bool = True,
        runtime_configs: dict | None = None,
        workdir: str | None = "/sandbox",
        security_policy: SecurityPolicy | None = None,
        default_timeout: float | None = None,
        execution_timeout: float | None = None,
        session_timeout: float | None = None,
        **kwargs: dict[str, Any],
    ) -> None:
        """Initialize Podman session."""
        config = SessionConfig(
            image=image,
            dockerfile=dockerfile,
            lang=SupportedLanguage(lang.upper()),
            verbose=verbose,
            workdir=workdir or "/sandbox",
            runtime_configs=runtime_configs or {},
            security_policy=security_policy,
            default_timeout=default_timeout,
            execution_timeout=execution_timeout,
            session_timeout=session_timeout,
        )

        # Initialize BaseSession (skip Docker's __init__)
        from llm_sandbox.core.session_base import BaseSession

        BaseSession.__init__(self, config=config, **kwargs)

        if not client:
            self._log("Using local Podman context since client is not provided.")
            self.client = PodmanClient.from_env()
        else:
            self.client = client

        self.container_api = PodmanContainerAPI(self.client, stream)

        # Set other attributes
        self.docker_image: Image
        self.keep_template: bool = keep_template
        self.commit_container: bool = commit_container
        self.is_create_template: bool = False
        self.stream: bool = stream

        if mounts:
            import warnings

            warnings.warn(
                "The 'mounts' parameter is deprecated and will be removed in a future version. "
                "Put the mounts in 'runtime_configs' instead.",
                DeprecationWarning,
                stacklevel=2,
            )
            self.config.runtime_configs.setdefault("mounts", []).append(mounts)

==> llm_sandbox-0.3.7-py3-none-any.whl: llm_sandbox/podman.py <==
# ruff: noqa: E501

from typing import TYPE_CHECKING, Any

from podman import PodmanClient

from llm_sandbox.const import SupportedLanguage
from llm_sandbox.core.config import SessionConfig
from llm_sandbox.docker import DockerContainerAPI, SandboxDockerSession
from llm_sandbox.security import SecurityPolicy

if TYPE_CHECKING:
    from podman.domain.images import Image


class PodmanContainerAPI(DockerContainerAPI):
    """Podman implementation of the ContainerAPI protocol."""

    def __init__(self, client: PodmanClient, stream: bool = True) -> None:
        """Initialize Podman container API."""
        self.client = client
        self.stream = stream

    def execute_command(self, container: Any, command: str, **kwargs: Any) -> tuple[int, Any]:
        """Execute command in Podman container."""
        workdir = kwargs.get("workdir")
        exec_kwargs: dict[str, Any] = {
            "cmd": command,
            "stream": self.stream,
            "tty": False,
            "stderr": True,
            "stdout": True,
            "demux": True,
        }
        if workdir:
            exec_kwargs["workdir"] = workdir

        result = container.exec_run(**exec_kwargs)
        # Podman returns (exit_code, output) tuple, Docker returns object with .exit_code and .output
        exit_code, output = result
        return exit_code or 0, output


class SandboxPodmanSession(SandboxDockerSession):
    r"""Sandbox session implemented using Podman containers.

    This class provides a sandboxed environment for code execution by leveraging Podman.
    It inherits from SandboxDockerSession since Podman is designed to be Docker-compatible,
    only overriding the differences in client initialization and API behavior.
    """

    def __init__(
        self,  # NOSONAR
        client: PodmanClient | None = None,
        image: str | None = None,
        dockerfile: str | None = None,
        lang: str = SupportedLanguage.PYTHON,
        keep_template: bool = False,
        commit_container: bool = False,
        verbose: bool = False,
        mounts: list | None = None,
        stream: bool = False,
        runtime_configs: dict | None = None,
        workdir: str | None = "/sandbox",
        security_policy: SecurityPolicy | None = None,
        default_timeout: float | None = None,
        execution_timeout: float | None = None,
        session_timeout: float | None = None,
        **kwargs: dict[str, Any],
    ) -> None:
        """Initialize Podman session."""
        config = SessionConfig(
            image=image,
            dockerfile=dockerfile,
            lang=SupportedLanguage(lang.upper()),
            verbose=verbose,
            workdir=workdir or "/sandbox",
            runtime_configs=runtime_configs or {},
            security_policy=security_policy,
            default_timeout=default_timeout,
            execution_timeout=execution_timeout,
            session_timeout=session_timeout,
        )

        # Initialize BaseSession (skip Docker's __init__)
        from llm_sandbox.core.session_base import BaseSession

        BaseSession.__init__(self, config=config, **kwargs)

        if not client:
            self._log("Using local Podman context since client is not provided.")
            self.client = PodmanClient.from_env()
        else:
            self.client = client

        self.container_api = PodmanContainerAPI(self.client, stream)

        # Set other attributes
        self.docker_image: Image
        self.keep_template: bool = keep_template
        self.commit_container: bool = commit_container
        self.is_create_template: bool = False
        self.stream: bool = stream

        if mounts:
            import warnings

            warnings.warn(
                "The 'mounts' parameter is deprecated and will be removed in a future version. "
                "Put the mounts in 'runtime_configs' instead.",
                DeprecationWarning,
                stacklevel=2,
            )
            self.config.runtime_configs.setdefault("mounts", []).append(mounts)
